-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Title       : signal_generator
-- Design      : Projekt_FIR
-- Author      : Jerzy Kasperek & Pawe J.Rajda
-- Company     : Katedra Elektroniki AGH
--
-------------------------------------------------------------------------------
-- Module signal_generator
-- virtual module responsibile for input data reading...
-- In real designs it could be for example an A/D converter interface....	
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;
-------------------------------------------------------------------------------
-- MATLAB interface libraries
library aldec;
use aldec.matlab.all;
-------------------------------------------------------------------------------
entity signal_generator is
	port(
		clk : in std_logic;
		rst : in std_logic;
		data_in : out std_logic_vector(15 downto 0)
		);
end signal_generator;
-------------------------------------------------------------------------------
architecture signal_generator of signal_generator is
	signal probka: std_logic_vector	(15 downto 0):= x"0000";
	
begin
	
	
	PUT_samples: 	process
		variable end_MATLAB: boolean:=false;
		variable init_MATLAB: boolean:=false;
		
		variable matrix_ID: integer;  
		variable how_many_samples: integer;
		variable data_pointer : TDims (1 to 2) := (1,1);
		variable data_sample: std_logic_vector(15 downto 0);
		
	begin
		
		-- MATLAB init, read how many samples in data input file 
		-- run this part of the code just once...
		if init_MATLAB = false then
			init_MATLAB := true;		
			-- execute MATLAB script
			eval_string("input_data_rd");
			matrix_ID := ml2hdl("input_data_matrix");
			how_many_samples := get_dim(matrix_ID, 2);
		end if;
		-- 	if no reset get data
		if rst = '1' then
			data_sample  := (others => '0');	
			data_pointer := (1,1);
		else	
			get_item(data_sample, matrix_ID, data_pointer );
			data_pointer(2):=data_pointer(2)+1;
			data_in <= data_sample;
		end if;	
		-- data end?
		if (data_pointer(2) > how_many_samples  and  init_MATLAB = true) then
			report "End input data"; 
			destroy_array(matrix_ID);
			-- process end...
			wait;
		end if;
		
		data_in <= data_sample;
		-- run on every rising edge
		wait until clk'event and clk ='1';
	end process;
	
	
end signal_generator;
