-------------------------------------------------------------------------------
--
-- Title       : signal_receiver
-- Design      : Projekt_FIR
-- Author      : Jerzy Kasperek & Pawe J.Rajda
-- Company     : Katedra Elektroniki AGH
--
-------------------------------------------------------------------------------
-- Module signal_receiver
-- virtual module responsibile for output data writing...
-- In real designs it could be for example an D/A converter interface....	
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;
-------------------------------------------------------------------------------
-- MATLAB interface libraries
library aldec;
use aldec.matlab.all;
-------------------------------------------------------------------------------
entity signal_receiver is
	port(
		clk : in std_logic;
		rst : in std_logic;
		data_out : in std_logic_vector(15 downto 0)
		);
end signal_receiver;
-------------------------------------------------------------------------------
architecture signal_receiver of signal_receiver is
	
	signal data_sample : std_logic_vector(15 downto 0);
begin  
	
	GET_samples:	process	 
		
		variable data_counter : integer:=1;
		variable init_MATLAB: boolean:=false;
		variable matrix_ID: integer;  
		variable how_many_samples: integer;
	begin
		-- MATLAB init, read how many samples in data input file 
		-- run this part of the code just once...
		-- of course can be also passed via testbench
		if init_MATLAB = false then
			init_MATLAB := true;	  
			-- execute MATLAB script
			eval_string("input_data_rd");
			matrix_ID := ml2hdl("input_data_matrix");
			how_many_samples := get_dim(matrix_ID, 2);
		end if;
		
		-- if no reset get samples
		if (rst = '1' or init_MATLAB = false )then
			data_counter := 1;
		else			
			if data_counter < (how_many_samples+1) then
				data_sample <= data_out;
				put_variable("i", data_counter);
				put_variable("signal_sample", data_sample, 0);
				-- execute MATLAB script simple data capture
				eval_string( "collect_samples" );
				data_counter:=data_counter+1;
			else
				-- Write output file
				-- execute MATLAB script 
				eval_string( "output_data_wr" );   
				-- process end...				
				wait;
			end if;
		end if;
		-- run on every rising edge
		wait until clk'event and clk ='1';		
	end process;
	
end signal_receiver;
